package asynctopic;

import javax.jms.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

 /**
 * AsynchTopicBeispiel demonstriert einen Message Listener im
 * publish/subscribe Modell.  Der Publisher published mehrere Messages und
 * der Subscriber liest diese asynchron.
 * <p>
 * Das Progrmam einthlt einen MultiplePublisher , einen AsynchSubscriber und
 * eine Listener Klasse, eine main Methode und eine Methode
 * welche den Subscriber und den Publisher Thread startet
 * <p>
 * Sie mssen ein Topic und eine "controlQueue" angeben,
 * Diese mssen Sie vorgngig eintragen
 */
public class AsynchTopicBeispiel {
    final String  CONTROL_QUEUE = "controlQueue";
    String        topicName = null;
    int           exitResult = 0;

    /**
     * AsynchSubscriber liest mehrere Messages aus dem Topic
     * (asynchron) mit dem Message Listener : TextListener.
     */
    public class AsynchSubscriber extends Thread {

        /**
         * TextListener implementiert das MessageListener Interface :
         * die onMessage Method wird fr die AsynchSubscriber Klasse
         * implementiert.
         *
         */
        private class TextListener implements MessageListener {
            final Hilfsklassen.DoneLatch  monitor =
                new Hilfsklassen.DoneLatch();

            /**
             * Casten der Message zu einer TextMessage und Anzeige des Textes.
             * Eine nicht-Text Message zeigt das Ende des Messagestromes an.
             *
             * @param message	ankommende Message
             */
            public void onMessage(Message message) {
                if (message instanceof TextMessage) {
                    TextMessage  msg = (TextMessage) message;

                    try {
                        System.out.println("[SUBSCRIBER THREAD]Message werden gelesen: "
                                           + msg.getText());
                    } catch (JMSException e) {
                        System.out.println("Exception in onMessage(): "
                                           + e.toString());
                    }
                } else {
                    monitor.allDone();
                }
            }
        }

        /**
         * Starten des Threads
         */
        public void run() {
            TopicConnectionFactory  topicConnectionFactory = null;
            TopicConnection         topicConnection = null;
            TopicSession            topicSession = null;
            Topic                   topic = null;
            TopicSubscriber         topicSubscriber = null;
            TextListener            topicListener = null;

            try {
                topicConnectionFactory =
                    Hilfsklassen.getTopicConnectionFactory();
                topicConnection =
                    topicConnectionFactory.createTopicConnection();
                topicSession = topicConnection.createTopicSession(false,
                    Session.AUTO_ACKNOWLEDGE);
                topic = Hilfsklassen.getTopic(topicName, topicSession);
            } catch (Exception e) {
                System.out.println("Probleme mit Connection: " + e.toString());
                if (topicConnection != null) {
                    try {
                        topicConnection.close();
                    } catch (JMSException ee) {}
                }
    	        System.exit(1);
            }

            /*
             * Ablauf:
             * kreiere Subscriber.
             * registriere Message Listener (TextListener).
             * starte Message Lieferung
             * sende synchronisierte Message an Publisher
             * warte bis alle Meldungen da sind
             * der Listener zeigt die ankommenden Meldungen an
             */
            try {
                topicSubscriber = topicSession.createSubscriber(topic);
                topicListener = new TextListener();
                topicSubscriber.setMessageListener(topicListener);
                topicConnection.start();

                // Let publisher know that subscriber is ready.
                try {
                    Hilfsklassen.sendSynchronizeMessage("[SUBSCRIBER THREAD] ",
                                                            CONTROL_QUEUE);
                } catch (Exception e) {
                    System.out.println("Queue fehlt vermutlich: " + e.toString());
                    if (topicConnection != null) {
                        try {
                            topicConnection.close();
                        } catch (JMSException ee) {}
                    }
    	            System.exit(1);
    	        }

                /*
                 * Asynchrones verarbeiten der Meldungen.
                 * Blockiere bis der Publisher eine Control Message sendet
                 */
                topicListener.monitor.waitTillDone();
            } catch (JMSException e) {
                System.out.println("Eine Exception trat auf: " + e.toString());
                exitResult = 1;
            } finally {
                if (topicConnection != null) {
                    try {
                        topicConnection.close();
                    } catch (JMSException e) {
                        exitResult = 1;
                    }
                }
            }
        }
    }

    /**
     * MultiplePublisher publiziert mehrere Messages an das Topic.
     */
    public class MultiplePublisher extends Thread {

        /**
         * Thread starten.
         */
        public void run() {
            TopicConnectionFactory  topicConnectionFactory = null;
            TopicConnection         topicConnection = null;
            TopicSession            topicSession = null;
            Topic                   topic = null;
            TopicPublisher          topicPublisher = null;
            TextMessage             message = null;
            final int               NUMMSGS = 20;
            final String            MSG_TEXT = new String("Hier ist die Message");

            try {
                topicConnectionFactory =
                    Hilfsklassen.getTopicConnectionFactory();
                topicConnection =
                    topicConnectionFactory.createTopicConnection();
                topicSession = topicConnection.createTopicSession(false,
                    Session.AUTO_ACKNOWLEDGE);
                topic = Hilfsklassen.getTopic(topicName, topicSession);
            } catch (Exception e) {
                System.out.println("Connection Problem: " + e.toString());
                if (topicConnection != null) {
                    try {
                        topicConnection.close();
                    } catch (JMSException ee) {}
                }
    	        System.exit(1);
            }

            /*
             * snychronisiere Publisher und Subscriber.
             * kreiere Text Message.
             * sende Messages
             * sende Ende-Message.
             * schliesse connection.
             */
            try {
                /*
                 * synchronisiere
                 */
                try {
                    Hilfsklassen.receiveSynchronizeMessages("[PUBLISHER THREAD]",
                                                              CONTROL_QUEUE, 1);
                } catch (Exception e) {
                    System.out.println("Queue fehlt wahrscheinlich: " + e.toString());
                    if (topicConnection != null) {
                        try {
                            topicConnection.close();
                        } catch (JMSException ee) {}
                    }
    	            System.exit(1);
    	        }

                topicPublisher = topicSession.createPublisher(topic);
                message = topicSession.createTextMessage();
                for (int i = 0; i < NUMMSGS; i++) {
                    message.setText(MSG_TEXT + " " + (i + 1));
                    System.out.println("[PUBLISHER THREAD]Publish Message: "
                        + message.getText());
                    topicPublisher.publish(message);
                }

                // Senden einer nicht-Text Control Message zeigt das Ende an
                topicPublisher.publish(topicSession.createMessage());
            } catch (JMSException e) {
                System.out.println("Eine Exception wurde geworfen: " + e.toString());
                exitResult = 1;
            } finally {
                if (topicConnection != null) {
                    try {
                        topicConnection.close();
                    } catch (JMSException e) {
                        exitResult = 1;
                    }
                }
            }
        }
    }

    /**
     * Instanzieren von Publisher und Subscriber Threads
     * Mit join() wird gewartet, bis die Threads beendet sind
     * <p>
     * Der Subscriber muss vor dem Publisher starten
     */
    public void run_threads() {
        AsynchSubscriber   asynchSubscriber = new AsynchSubscriber();
        MultiplePublisher  multiplePublisher = new MultiplePublisher();

        multiplePublisher.start();
        asynchSubscriber.start();
        try {
            asynchSubscriber.join();
            multiplePublisher.join();
        } catch (InterruptedException e) {}
    }

    /**
     * Lesen des Topis aus der Kommandozeile
     * run_threads Methode startet die Programmthreads
     *
     * @param args	Topic
     */
    public static void main(String[] args) {
        AsynchTopicBeispiel  ate = new AsynchTopicBeispiel();

        if (args.length != 1) {
    	    System.out.println("Usage: java AsynchTopicBeispiel <topic_name>");
    	    //System.exit(1);
            ate.topicName = "meinTopic";
    	} else {
        ate.topicName = new String(args[0]); }
        System.out.println("Name des Topics " + ate.topicName);

    	ate.run_threads();
    	Hilfsklassen.exit(ate.exitResult);
    }
}
