package durablesubscriber;

import javax.jms.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

/**
 * DurableSubscriber
 * zeigt die durable Subscription
 * diese ist auch aktiv, wenn der Subscriber nicht aktiv ist
 * <p>
 * Aufbau:
 * Klassen : DurableSubscriber , MultiplePublisher
 * ;ethoden :main plus eine Methode zum Instanzieren der Threads
 * <p>
 * Ablauf:
 * Subscriber startet
 * Publisher publiziert
 * <p>
 * Der Subscriber unterbricht
 * Der Publisher sendet weiter
 * Der Subscriber startet neu und liest die inzwischen eingetroffenen Messages
 * <p>
 * Eingabe:
 * ein Topic auf der Kommandozeile (sonst wird der Standardwert genommen: meinTopic)
 */
public class DurableSubscriberBeispiel {
    String      topicName = null;
    int         exitResult = 0;
    static int  startindex = 0;

    /**
     * DurableSubscriber
     * Konstruktor,  startSubscriber(), closeSubscriber() und finish() Methoden.
     * <p>
     * Die Klasse liest die Messages asynchron mit dem Message Listener
     * TextListener.
     */
    public class DurableSubscriber {
        TopicConnection  topicConnection = null;
        TopicSession     topicSession = null;
        Topic            topic = null;
        TopicSubscriber  topicSubscriber = null;
        TextListener     topicListener = null;

        /**
         * TextListener implementiert das MessageListener Interface :
         * onMessage Methode fr DurableSubscriber.
         */
        private class TextListener implements MessageListener {
            final Hilfsklassen.DoneLatch  monitor =
                new Hilfsklassen.DoneLatch();

            /**
             * Casten der Message zu einer TextMessage und Anzeige des Textes
             * eine nicht-Text Message beendet das Ganze
             *
             * @param message	eintreffende Message
             */
            public void onMessage(Message message) {
                if (message instanceof TextMessage) {
                    TextMessage  msg = (TextMessage) message;

                    try {
                        System.out.println("[SUBSCRIBER]Lesen der Messages: "
                                           + msg.getText());
                    } catch (JMSException e) {
                        System.out.println("Exception in onMessage(): "
                                           + e.toString());
                    }
                } else {
                    monitor.allDone();
                }
            }
        }

        /**
         * Konstruktor: lookup einer Connection Factory und eines Topics
         * kreiert ein connection und ein session Objekt
         */
        public DurableSubscriber() {
            TopicConnectionFactory  topicConnectionFactory = null;

            try {
                topicConnectionFactory =
                    Hilfsklassen.getTopicConnectionFactory();
                topicConnection =
                    topicConnectionFactory.createTopicConnection();
                topicConnection.setClientID("DurableSubscriberBeispiel");
                topicSession = topicConnection.createTopicSession(false,
                    Session.AUTO_ACKNOWLEDGE);
                topic = Hilfsklassen.getTopic(topicName, topicSession);
            } catch (Exception e) {
                System.out.println("Connection Problem: " + e.toString());
                if (topicConnection != null) {
                    try {
                        topicConnection.close();
                    } catch (JMSException ee) {}
                }
    	        System.exit(1);
            }
        }

        /**
         * Stopen der Connection,
         * Kreiern eines durable Subscriber, registrieren des Message
         * Listeners (TextListener)
         * Starts der Message Delivery
         * Listener: teigt die erhaltenenen MessageTexte an
         */
        public void startSubscriber() {
            try {
                System.out.println("Starte Subscriber");
                topicConnection.stop();
                topicSubscriber = topicSession.createDurableSubscriber(topic,
                    "DasWars");
                topicListener = new TextListener();
                topicSubscriber.setMessageListener(topicListener);
                topicConnection.start();
            } catch (JMSException e) {
                System.out.println("Eine Exception trat auf: " + e.toString());
                exitResult = 1;
            }
        }

        /**
         * Warte auf Subscriber: dieser muss zuerst gestartet werden
         */
        public void closeSubscriber() {
            try {
                topicListener.monitor.waitTillDone();
                System.out.println("Schliessen des Subscriber");
                topicSubscriber.close();
            } catch (JMSException e) {
                System.out.println("Eine Exception trat auf: " + e.toString());
                exitResult = 1;
            }
        }

        /**
         * Clode Connection.
         */
        public void finish() {
            if (topicConnection != null) {
                try {
                    topicSession.unsubscribe("DasWars");
                    topicConnection.close();
                } catch (JMSException e) {
                    exitResult = 1;
                }
            }
        }
    }

    /**
     * MultiplePublisher
     * Publiziert mehrere Messages an ein Topic
     * enthlt:
     * Konstruktor, publishMessages und finish Methode.
     */
    public class MultiplePublisher {
        TopicConnection  topicConnection = null;
        TopicSession     topicSession = null;
        Topic            topic = null;
        TopicPublisher   topicPublisher = null;

        /**
         * Konstruktor: sucht Connection Factory u
         * kreiert connection und session Objekte
         * Kreiert den Publisher.
         */
        public MultiplePublisher() {
            TopicConnectionFactory  topicConnectionFactory = null;

            try {
                topicConnectionFactory =
                    Hilfsklassen.getTopicConnectionFactory();
                topicConnection =
                    topicConnectionFactory.createTopicConnection();
                topicSession = topicConnection.createTopicSession(false,
                    Session.AUTO_ACKNOWLEDGE);
                topic = Hilfsklassen.getTopic(topicName, topicSession);
                topicPublisher = topicSession.createPublisher(topic);
            } catch (Exception e) {
                System.out.println("Connection Problem: " + e.toString());
                if (topicConnection != null) {
                    try {
                        topicConnection.close();
                    } catch (JMSException ee) {}
                }
    	        System.exit(1);
            }
        }

        /**
         * Kreiere Text Message
         * Sende Text Messages
         * Messages mssen persistent sein
         */
        public void publishMessages() {
            TextMessage   message = null;
            int           i;
            final int     NUMMSGS = 3;
            final String  MSG_TEXT = new String("Hier kommt Meldung_");

            try {
                message = topicSession.createTextMessage();
                for (i = startindex; i < startindex + NUMMSGS; i++) {
                    message.setText(MSG_TEXT +  (i + 1));
                    System.out.println("[PUBLISHER]Message "
                        + message.getText()+" wird publiziert!");
                    topicPublisher.publish(message);
                }

                // Abschluss mit nicht-Text Message
                topicPublisher.publish(topicSession.createMessage());
                startindex = i;
            } catch (JMSException e) {
                System.out.println("Eine Exception trat auf: " + e.toString());
                exitResult = 1;
            }
        }

        /**
         * Schliesse connection.
         */
        public void finish() {
            if (topicConnection != null) {
                try {
                    topicConnection.close();
                } catch (JMSException e) {
                    exitResult = 1;
                }
            }
        }
    }

    /**
     * Instanziere:
     * Publisher, Subscriber Klasse
     *
     * Starts
     * Subscriber;
     * Publisher publiziert einige Messages
     *
     * Scliessen des Subscribers, der Publisher fhrt fort
     *
     * Restart des Subscribers, lesen der Messages
     *
     * Close Connection
     */
    public void run_program() {
        DurableSubscriber  durableSubscriber = new DurableSubscriber();
        MultiplePublisher  multiplePublisher = new MultiplePublisher();

        durableSubscriber.startSubscriber();
        multiplePublisher.publishMessages();
        durableSubscriber.closeSubscriber();
        multiplePublisher.publishMessages();
        durableSubscriber.startSubscriber();
        durableSubscriber.closeSubscriber();
        multiplePublisher.finish();
        durableSubscriber.finish();
    }

    /**
     * Lies Topic und starte
     *
     * @param args	Topic
     */
    public static void main(String[] args) {
        DurableSubscriberBeispiel  dse = new DurableSubscriberBeispiel();

        if (args.length != 1) {
    	    System.out.println("Usage: java DurableSubscriberBeispiel <topic_name>");
    	    //System.exit(1);
            dse.topicName = new String("meinTopic");
    	} else { dse.topicName = new String(args[0]);}
        System.out.println("Topic Name ist " + dse.topicName);

    	dse.run_program();
    	Hilfsklassen.exit(dse.exitResult);
    }
}
