package messageheader;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

 import java.sql.*;
import java.util.*;
import javax.jms.*;

/**
 * Message Header Felder.
 * <p>
 * HeaderPublisher , HeaderSubscriber:
 * die display_headers() Methode zeigt die Header Felder an.
 * Es wird ein Publisher und ein Subscriber implementiert, mit Threads
 * <p>
 * Der Publisher sendet x Packete an den Subscriber
 * Das Programm liest den Header bevor die Message gesendet bzw. gleich nachdem
 * sie empfangen wird.
 * <p>
 * Das Topis steht auf der Kommandozeile
 * pDas Programm verwendet eine "controlQueue", zur Kontrolle des Endes
 */
public class MessageHeadersTopic {
    final String  CONTROL_QUEUE = "controlQueue";
    String        topicName = null;
    int           exitResult = 0;

    /**
     * HeaderPublisher
     * schreibt x Messages, setzt die JMSType Header Felder
     * (JMSCorrelationID und JMSReplyTo werden im Beispiel RequestReplyQueue
     * gezeigt). example.)
     * Setzen der Client Property "messageNumber".
     *
     * Die Methode displayHeaders wird vor der publish Methode aufgerufen
     */
    public class HeaderPublisher extends Thread {

        /**
         * Starte den Thread
         */
        public void run() {
            TopicConnectionFactory  topicConnectionFactory = null;
            TopicConnection         topicConnection = null;
            TopicSession            topicSession = null;
            Topic                   topic = null;
            TopicPublisher          topicPublisher = null;
            TextMessage             message = null;
            final String            MSG_TEXT = new String("Message Objekt fuer die Header Analyse");

            try {
                topicConnectionFactory =
                    Hilfsklassen.getTopicConnectionFactory();
                topicConnection =
                    topicConnectionFactory.createTopicConnection();
                topicSession = topicConnection.createTopicSession(false,
                    Session.AUTO_ACKNOWLEDGE);
                topic = Hilfsklassen.getTopic(topicName, topicSession);
            } catch (Exception e) {
                System.out.println("[HeaderPublisher]Connection Problem: " + e.toString());
                if (topicConnection != null) {
                    try {
                        topicConnection.close();
                    } catch (JMSException ee) {}
                }
    	        System.exit(1);
            }

            try {
                /*
                 * Synchronisiere mit dem Subscriber.
                 * Warte bis angezeigt wird,
                 * dass der Subscriber bereit ist Messages zu empfangen
                 */
                try {
                    Hilfsklassen.receiveSynchronizeMessages("PUBLISHER THREAD: ",
                                                              CONTROL_QUEUE, 1);
                } catch (Exception e) {
                    System.out.println("[HeaderPublisher]Queue wurde vermutlich nicht definiert: " + e.toString());
                    if (topicConnection != null) {
                        try {
                            topicConnection.close();
                        } catch (JMSException ee) {}
                    }
    	            System.exit(1);
    	        }

                // Kreiere den Publisher.
                topicPublisher = topicSession.createPublisher(topic);

                // Erste Message: publish ohne Argument
                message = topicSession.createTextMessage();
                message.setJMSType("Einfach");
                System.out.println("[PUBLISHER THREAD]Setze JMSType auf "
                    + message.getJMSType());
                message.setIntProperty("messageNummer", 1);
                System.out.println("[PUBLISHER THREAD]Setze Client Property messageNummber auf "
                    + message.getIntProperty("messageNummer"));
                message.setText(MSG_TEXT);
                System.out.println("[PUBLISHER THREAD]Setze den Message Text auf: "
                    + message.getText());
                System.out.println("[PUBLISHER THREAD]Headers vor dem Publizieren:");
                displayHeaders(message, "[PUBLISHER THREAD]");
                topicPublisher.publish(message);

                /*
                 * zweite Message: 3-Argumente in der publish() Methode
                 * Delivery Mode, Priority und Expiration
                 */
                message = topicSession.createTextMessage();
                message.setJMSType("Weniger Einfach");
                System.out.println("\n[PUBLISHER THREAD]Setze JMSType auf "
                    + message.getJMSType());
                message.setIntProperty("messageNummer", 2);
                System.out.println("[PUBLISHER THREAD]Setze Client Property messageNummer auf "
                    + message.getIntProperty("messageNummer"));
                message.setText(MSG_TEXT + " Nochmal");
                System.out.println("[PUBLISHER THREAD]Setze Message Text auf: "
                    + message.getText());
                displayHeaders(message, "[PUBLISHER THREAD] ");
                topicPublisher.publish(message, DeliveryMode.NON_PERSISTENT,
                    3, 10000);

                /*
                 * dritte Message: publish Methode ohne Argument
                 * MessageID und Timestamp disabled
                 */
                message = topicSession.createTextMessage();
                message.setJMSType("Disable Test");
                System.out.println("\n[PUBLISHER THREAD]Setze JMSType auf "
                    + message.getJMSType());
                message.setIntProperty("messageNummer", 3);
                System.out.println("[PUBLISHER THREAD]Setze Client Property messageNummer auf "
                    + message.getIntProperty("messageNummer"));
                message.setText(MSG_TEXT
                    + " MessageID und Timestamp sind disabled");
                System.out.println("[PUBLISHER THREAD]Setze den Message Text auf: "
                    + message.getText());
                topicPublisher.setDisableMessageID(true);
                topicPublisher.setDisableMessageTimestamp(true);
                System.out.println("[PUBLISHER THREAD]Disable Message ID und Timestamp");
                displayHeaders(message, "[PUBLISHER THREAD] ");
                topicPublisher.publish(message);
            } catch (JMSException e) {
                System.out.println("Eine Exception wurde geworfen: " + e.toString());
                exitResult = 1;
            } finally {
                if (topicConnection != null) {
                    try {
                        topicConnection.close();
                    } catch (JMSException e) {
                        exitResult = 1;
                    }
                }
            }
        }
    }

    /**
     * HeaderSubscriber:
     * empfngt die Messages und ruft
     * displayHeaders auf, um aufzuzeigen, wie das Publizieren die Header Werte
     * verndert hat
     * <p>
     * Erste Message (ohne Setzen von Feldern)
     * zeigt die Default Werte
     * <p>
     * Zweite Masseg (gesetzte Parameter)
     * zeigt die Werte, die von der publish() Methode gesetzt werden
     * <p>
     * Dritte Message (disable MessageID und Timestamp)
     * zeigt das Verhalten abhngig von der zugrundeliegeenden JMS Impl.
     *
     */
    public class HeaderSubscriber extends Thread {

        /**
         * Thread starten
         */
        public void run() {
            TopicConnectionFactory  topicConnectionFactory = null;
            TopicConnection         topicConnection = null;
            TopicSession            topicSession = null;
            Topic                   topic = null;
            TopicSubscriber         topicSubscriber = null;
            final boolean           NOLOCAL = true;
            TextMessage             message = null;

            try {
                topicConnectionFactory =
                    Hilfsklassen.getTopicConnectionFactory();
                topicConnection =
                    topicConnectionFactory.createTopicConnection();
                topicSession = topicConnection.createTopicSession(false,
                    Session.AUTO_ACKNOWLEDGE);
                topic = Hilfsklassen.getTopic(topicName, topicSession);
            } catch (Exception e) {
                System.out.println("Connection Problem: " + e.toString());
                if (topicConnection != null) {
                    try {
                        topicConnection.close();
                    } catch (JMSException ee) {}
                }
    	        System.exit(1);
            }

            /*
             * kreiere Subscriber, starte Message Delivery
             * Sende synchronize Message an Publisher.
             * Empfange die drei Messages
             * Rufe displayHeaders Methode auf um HeaderInfos anzuzeigen
             */
            try {
                topicSubscriber =
                    topicSession.createSubscriber(topic, null, NOLOCAL);
                topicConnection.start();

                // Subscriber an Publisher: ich bin bereit
                try {
                    Hilfsklassen.sendSynchronizeMessage("[SUBSCRIBER THREAD] ",
                                                            CONTROL_QUEUE);
                } catch (Exception e) {
                    System.out.println("[SUBSCRIBER THREAD]Queue fehlt vermutlich: " + e.toString());
                    if (topicConnection != null) {
                        try {
                            topicConnection.close();
                        } catch (JMSException ee) {}
                    }
    	            System.exit(1);
    	        }

                for (int i = 0; i < 3; i++) {
                    message = (TextMessage) topicSubscriber.receive();
                    System.out.println("\n[SUBSCRIBER THREAD]Message empfangen: "
                        + message.getText());
                    System.out.println("[SUBSCRIBER THREAD]Headers nach dem Message Empfang:");
                    displayHeaders(message, "[SUBSCRIBER THREAD] ");
                }
            } catch (JMSException e) {
                System.out.println("[SUBSCRIBER THREAD]Es ist eine Exception aufgetreten: " + e.toString());
                exitResult = 1;
            } finally {
                if (topicConnection != null) {
                    try {
                        topicConnection.close();
                    } catch (JMSException e) {
                        exitResult = 1;
                    }
                }
            }
        }
    }

    /**
     * Anzeige der Header Info (try catch damit alles angezeigt wird)
     *
     * @param message	Message, deren Header angezeigt werden soll
     * @param prefix	Prfix (Publisher oder Subscriber)
     */
    public void displayHeaders (Message message, String prefix) {
        Destination  dest = null;
        int          delMode = 0;
        long         expiration = 0;
        Time         expTime = null;
        int          priority = 0;
        String       msgID = null;
        long         timestamp = 0;
        Time         timestampTime = null;
        String       correlID = null;
        Destination  replyTo = null;
        boolean      redelivered = false;
        String       type = null;
        String       propertyName = null;

        try {
            System.out.println(prefix + "Headers aus der publish/send Methode: ");

            // Destination anzeigen (Topic).
            try {
                dest = message.getJMSDestination();
                System.out.println(prefix + " JMSDestination: " + dest);
            } catch (Exception e) {
                System.out.println(prefix + "Eine Exception trat auf: "
                    + e.toString());
                exitResult = 1;
            }

            // Delivery Modus anzeigen
            try {
                delMode = message.getJMSDeliveryMode();
                System.out.print(prefix);
                if (delMode == DeliveryMode.NON_PERSISTENT) {
                    System.out.println(" JMSDeliveryMode: non-persistent");
                } else if (delMode == DeliveryMode.PERSISTENT) {
                    System.out.println(" JMSDeliveryMode: persistent");
                } else {
                    System.out.println(" JMSDeliveryMode: weder persistent noch non-persistent; Fehler");
                }
            } catch (Exception e) {
                System.out.println(prefix + "Eine Exception trat ein: "
                    + e.toString());
                exitResult = 1;
            }

            /*
             * Anzeige der Expiration Time. Falls 0 (default),
             * expires die Message nie.  Sonst muss der Wert gecastet werden
             * um ein Time Objekt zu erhalten und anzuzeigen
             */
            try {
                expiration = message.getJMSExpiration();
                System.out.print(prefix);
                if (expiration != 0) {
                    expTime = new Time(expiration);
                    System.out.println(" JMSExpiration: " + expTime);
                } else {
                    System.out.println(" JMSExpiration: " + expiration);
                }
            } catch (Exception e) {
                System.out.println(prefix + "Eine Exception trat auf: "
                    + e.toString());
                exitResult = 1;
            }

            // Anzeige der Prioritt.
            try {
                priority = message.getJMSPriority();
                System.out.println(prefix + " JMSPriority: " + priority);
            } catch (Exception e) {
                System.out.println(prefix + "Eine Exception trat auf: "
                    + e.toString());
                exitResult = 1;
            }

            // Anzeige der Message ID.
            try {
                msgID = message.getJMSMessageID();
                System.out.println(prefix + " JMSMessageID: " + msgID);
            } catch (Exception e) {
                System.out.println(prefix + "Eine Exception trat auf: "
                    + e.toString());
                exitResult = 1;
            }

            /*
             * Anzeige des timestamp.
             * falls nicht 0, casten in ein Time Objekt
             */
            try {
                timestamp = message.getJMSTimestamp();
                System.out.print(prefix);
                if (timestamp != 0) {
                    timestampTime = new Time(timestamp);
                    System.out.println(" JMSTimestamp: " + timestampTime);
                } else {
                    System.out.println(" JMSTimestamp: " + timestamp);
                }
            } catch (Exception e) {
                System.out.println(prefix + "Eine Exception trat auf: "
                    + e.toString());
                exitResult = 1;
            }

            // Anzeige der correlationID.
            try {
                correlID = message.getJMSCorrelationID();
                System.out.println(prefix + " JMSCorrelationID: " + correlID);
            } catch (Exception e) {
                System.out.println(prefix + "Eine Exception trat auf: "
                    + e.toString());
                exitResult = 1;
            }

           // Anzeige der ReplyTo Destination.
           try {
                replyTo = message.getJMSReplyTo();
                System.out.println(prefix + " JMSReplyTo: " + replyTo);
            } catch (Exception e) {
                System.out.println(prefix + "Eine Exception trat auf: "
                    + e.toString());
                exitResult = 1;
            }

            // Anzeige des Redelivered Wert (normalerweise false).
            System.out.println(prefix + "Header durch JMS Provider gesetzt:");
            try {
                redelivered = message.getJMSRedelivered();
                System.out.println(prefix + " JMSRedelivered: " + redelivered);
            } catch (Exception e) {
                System.out.println(prefix + "Eine Exception trat auf: "
                    + e.toString());
                exitResult = 1;
            }

            // Anzeige des JMSType.
            System.out.println(prefix + "Headers vom Client Programm gesetzt:");
            try {
                type = message.getJMSType();
                System.out.println(prefix + " JMSType: " + type);
            } catch (Exception e) {
                System.out.println(prefix + "Eine Exception trat auf: "
                    + e.toString());
                exitResult = 1;
            }

            // Anzeige der Client Properties.
            try {
                for (Enumeration e = message.getPropertyNames(); e.hasMoreElements() ;) {
                    propertyName = new String((String) e.nextElement());
                    System.out.println(prefix + " Client Property "
                        + propertyName + ": "
                        + message.getObjectProperty(propertyName));
                }
            } catch (Exception e) {
                System.out.println(prefix + "Eine Exception trat auf: "
                    + e.toString());
                exitResult = 1;
            }
        } catch (Exception e) {
                System.out.println(prefix + "Eine Exception trat auf: "
                    + e.toString());
            exitResult = 1;
        }
    }

    /**
     * Instanzieren der Publisher und Subscriber Klassen
     * und starten der Threads
     * Join() sorgt dafr, dass die Thread beim Beenden aufeinander warten
     * <p>
     * Der Subscriber muss vor dem Publisher gestartet werden
     * Der Subscriber empfngt Messages normalerweise nur wenn er aktiv ist
     */
    public void run_threads() {
        HeaderSubscriber  headerSubscriber = new HeaderSubscriber();
        HeaderPublisher   headerPublisher = new HeaderPublisher();

        headerSubscriber.start();
        headerPublisher.start();
        try {
            headerSubscriber.join();
            headerPublisher.join();
        } catch (InterruptedException e) {}
    }

    /**
     * Lesen des Topis aus der Kommandozeile
     * run_threads fhrt Thread Programme aus
     *
     * @param args	Topic
     */
    public static void main(String[] args) {
        MessageHeadersTopic  mht = new MessageHeadersTopic();

        if (args.length != 1) {
    	    System.out.println("Usage: java MessageHeadersTopic <topic_name>");
            mht.topicName = new String("meinTopic");
    	    //System.exit(1);
    	} else {
        mht.topicName = new String(args[0]); }
        System.out.println("Der Name des Topic ist " + mht.topicName);

    	mht.run_threads();
    	Hilfsklassen.exit(mht.exitResult);
    }
}
