package messagekonversion;

import javax.jms.*;
import javax.naming.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

 /**
 * Hilfsprogramm / Klassen fr die Demo Programme
 * <p>
 * Setzen Sie <code>USE_JNDI</code> auf true oder false je nachdem
 * ob der Proviedr JNDI einsetzt.
 * <p>
 * Die Hilfsklassen umfassen folgende Methoden:
 * <ul>
 *   <li> getQueueConnectionFactory
 *   <li> getTopicConnectionFactory
 *   <li> getQueue
 *   <li> getTopic
 *   <li> jndiLookup
 *   <li> exit
 *   <li> receiveSynchronizeMessages
 *   <li> sendSynchronizeMessages
 * </ul>
 *
 * Die enthaltene Klasse DoneLatch enthlt folgende Methoden:
 * <ul>
 *   <li> waitTillDone
 *   <li> allDone
 * </ul>
 *
 */
public class Hilfsklassen {
    /*
     Definition der Systemeigenschaft "USE_JNDI" true falls JNDI lookup() verwendet wird.
     e.g. $JAVA -DUSE_JNDI=true ....
    */
    public static final boolean USE_JNDI = Boolean.getBoolean("USE_JNDI");
    /*
     Prfix fr JNDI lookup
     */
    public static final String jndiNamePrefix = "cn=";

    public static final String  QUEUECONFAC = "QueueConnectionFactory";
    public static final String  TOPICCONFAC = "TopicConnectionFactory";
    private static Context      jndiContext = null;

    /**
     * Liefert ein QueueConnectionFactory Objekt.
     * Falls der Provider JNDI einsetzt, dient diese Klasse als Wrapper
     * um die JNDI Methoden.
     * Falls der Provider nicht JNDI einsetzt, muss die Klasse ergnzt werden
     *
     * @return		ein QueueConnectionFactory Objekt
     * @throws		javax.naming.NamingException
     *                  (oder eine andere Exception)
     *                   falls der Name nicht gefunden wird
     */
    public static javax.jms.QueueConnectionFactory getQueueConnectionFactory()
      throws Exception {
        if (USE_JNDI) {
            return (javax.jms.QueueConnectionFactory) jndiLookup(QUEUECONFAC);
        } else {
            // liefere eine neue Provider-spezifische QueueConnectionFactory
            return new com.sun.messaging.QueueConnectionFactory();
        }
    }

    /**
     * Liefert ein TopicConnectionFactory Objekt
     * Falls der Provider JNDI, dann dient die Factory als Wrapper
     * um die jndiLookup Methode
     * Sonst mssen Sie die Klasse ergnzen!
     *
     * @return		ein TopicConnectionFactory Objekt
     * @throws		javax.naming.NamingException (oder sonst was)
     *                  falls der Name nicht gefunden wird
     */
    public static javax.jms.TopicConnectionFactory getTopicConnectionFactory()
      throws Exception {
        if (USE_JNDI) {
            return (javax.jms.TopicConnectionFactory) jndiLookup(TOPICCONFAC);
        } else {
            // return neue provider-spezifische TopicConnectionFactory
            return new com.sun.messaging.TopicConnectionFactory();
        }
    }

    /**
     * Liefert ein Queue Objekt.
     * Falls der Provider JNDI einsetzt, dient diese Klasse als Wrappen
     * um jndiLookup
     * Falls der Provider nicht JNDI einsetzt, dann mssen Sie die Klasse
     * durch anbieterspezifischen Code ergnzen
     *
     * @param name      String, Name der Warteschlange
     * @param session   ein QueueSession Objekt
     *
     * @return		Ein Queue Objekt
     * @throws		javax.naming.NamingException (oder sonst was)
     *                  falls der Name nicht gefunden wird.
     */
    public static javax.jms.Queue getQueue(String name,
                                           javax.jms.QueueSession session)
      throws Exception {
        if (USE_JNDI) {
            return (javax.jms.Queue) jndiLookup(name);
        } else {
            return session.createQueue(name);
        }
    }

    /**
     * Liefert ein Topic Objekt.
     * Falls der Provider JNDI einsetzt, dient diese Klasse als Wrappen
     * um jndiLookup
     * Falls der Provider nicht JNDI einsetzt, dann mssen Sie die Klasse
     * durch anbieterspezifischen Code ergnzen
     *
     * @param name      String mit dem Topic Namen
     * @param session   eine TopicSession Objekt
     *
     * @return		ein Topic Objekt
     * @throws		javax.naming.NamingException (oder sonst was)
     *                  falls der Name nicht gefunden wird
     */
    public static javax.jms.Topic getTopic(String name,
                                           javax.jms.TopicSession session)
      throws Exception {
        if (USE_JNDI) {
            return (javax.jms.Topic) jndiLookup(name);
        } else {
            return session.createTopic(name);
        }
    }

    /**
     * Kreiert ein JNDI InitialContext Objekt, falls dieses noch nicht existiert
     * Dann wird mit dem String das Objekt gesucht
     *
     * @param name	Name des gesuchten Objekts
     *
     * @return		an den <code>name</code> gebundenes Objekt
     * @throws		javax.naming.NamingException falls name nicht vorhanden
     */
    public static Object jndiLookup(String name) throws NamingException {
        Object    obj = null;

        if (jndiContext == null) {
            try {
                jndiContext = new InitialContext();
            } catch (NamingException e) {
                System.out.println("Fehler beim Kreieren des JNDI Context: " +
                    e.toString());
                throw e;
            }
        }
        try {
           obj = jndiContext.lookup(jndiNamePrefix + name);
        } catch (NamingException e) {
            System.out.println("JNDI lookup schlug fehl:" + name + ": " + e.toString());
            throw e;
        }
        return obj;
    }

    /**
     * Calls System.exit().
     *
     * @param result	Exit result; 0 zeigt fehlerfrei an
     */
    public static void exit(int result) {
        System.exit(result);
    }

    /**
     * Warte auf 'count' Messages an der controlQueue bevor weitergefahren wird.
     * Wird vom Publisher aufgerufen, um sicher zu sein, dass
     * Subscribers gestartet wurden bevor die Meldungen publiziert werden.
     * <p>
     * Falls controlQueue nicht exist wird eine EXception geworfen
     *
     * @param prefix	prefix (Publisher oder Subscriber) welcher angezeigt wird
     * @param controlQueueName	Name der control queue
     * @param count	Anzahl erhaltener Messages
     */
    public static void receiveSynchronizeMessages(String prefix,
                                                  String controlQueueName,
                                                  int count)
      throws Exception {
        QueueConnectionFactory  queueConnectionFactory = null;
        QueueConnection         queueConnection = null;
        QueueSession            queueSession = null;
        Queue                   controlQueue = null;
        QueueReceiver           queueReceiver = null;

        try {
            queueConnectionFactory =
                Hilfsklassen.getQueueConnectionFactory();
            queueConnection = queueConnectionFactory.createQueueConnection();
            queueSession = queueConnection.createQueueSession(false,
                                                 Session.AUTO_ACKNOWLEDGE);
            controlQueue = getQueue(controlQueueName, queueSession);
            queueConnection.start();
        } catch (Exception e) {
            System.out.println("Probleme beim Verbindungsaufbau: " + e.toString());
            if (queueConnection != null) {
                try {
                    queueConnection.close();
                } catch (JMSException ee) {}
            }
            throw e;
        }

        try {
            System.out.println(prefix + "Synchronisierte Messages von "
                                + controlQueueName +
                                " werden empfangen; Anzahl = " + count);
            queueReceiver = queueSession.createReceiver(controlQueue);
            while (count > 0) {
                queueReceiver.receive();
                count--;
                System.out.println(prefix
                 + "Synchronisierte Message wurde empfangen; es werden noch "
                 + count + " weitere erwartet");
            }
        } catch (JMSException e) {
            System.out.println("Exception: " + e.toString());
            throw e;
        } finally {
            if (queueConnection != null) {
                try {
                    queueConnection.close();
                } catch (JMSException e) {}
            }
        }
    }

    /**
     * Senden einer Message an die controlQueue.
     * Aufruf durch den Subscriber, um den Publisher zu informieren
     * dass er bereits ist, Meldungen zu empfangen.
     * <p>
     * Falls die controlQueue nicht existiert
     * soll eine Exception geworfen werden.
     *
     * @param prefix	prefix (Publisher oder Subscriber)
     *                  der angezeigt werden soll
     * @param controlQueueName	Name der Kontrollwarteschlange
     */
    public static void sendSynchronizeMessage(String prefix,
                                              String controlQueueName)
      throws Exception {
        QueueConnectionFactory  queueConnectionFactory = null;
        QueueConnection         queueConnection = null;
        QueueSession            queueSession = null;
        Queue                   controlQueue = null;
        QueueSender             queueSender = null;
        TextMessage             message = null;

        try {
            queueConnectionFactory =
                Hilfsklassen.getQueueConnectionFactory();
            queueConnection = queueConnectionFactory.createQueueConnection();
            queueSession = queueConnection.createQueueSession(false,
                                                 Session.AUTO_ACKNOWLEDGE);
            controlQueue = getQueue(controlQueueName, queueSession);
        } catch (Exception e) {
            System.out.println("Problem beim Verbindungsaufbau: " + e.toString());
            if (queueConnection != null) {
                try {
                    queueConnection.close();
                } catch (JMSException ee) {}
            }
            throw e;
        }

        try {
            queueSender = queueSession.createSender(controlQueue);
            message = queueSession.createTextMessage();
            message.setText("synchronize");
            System.out.println(prefix + "Senden einer synchronisierten Message an "
                               + controlQueueName);
            queueSender.send(message);
        } catch (JMSException e) {
            System.out.println("Exception: " + e.toString());
            throw e;
        } finally {
            if (queueConnection != null) {
                try {
                    queueConnection.close();
                } catch (JMSException e) {}
            }
        }
    }

    /**
     * Monitor Klasse fr die asynchronen Beispiele.
     * Der Producer signalisiert das Ende des Message Streams
     * Der Listener ruft allDone() auf, um den Konsumenten zu informieren,
     * dass das Signal angekommen ist.
     * Der Konsument ruft waitTillDone() auf, um auf dieses Signal zu warten
     *
     */
    static public class DoneLatch {
        boolean  done = false;

        /**
         * Wartet, bis done true wird
         */
        public void waitTillDone() {
            synchronized (this) {
                while (! done) {
                    try {
                        this.wait();
                    } catch (InterruptedException ie) {}
                }
            }
        }

        /**
         * Setz done auf true.
         */
        public void allDone() {
            synchronized (this) {
                done = true;
                this.notify();
            }
        }
    }
}
