package topicselector;

import java.util.*;
import javax.jms.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

 /**
 * TopicSelectors
 * zeigt den Einsatz mehrerer Subscribers und Message Selectoren.
 * <p>
 * Aufbau:
 * Publisher , Subscriber mit Listener
 * main und einer Methode die  Subscriber und Publisher Threads startet
 * <p>
 * Ablauf:
 * Der Publisher publiziert 30 Messages (6 unterschiedliche Typen, zufallsausgewhlt)
 * Abschluss mit einer "Ende" Message.
 * Es werden vier Instanzen der Subscriber Klasse gebildet
 * unterschiedlichen Typs, plus Ende Klasse
 * Jeder Subscriber benutzt einen anderen Selektor
 * Publisher und Subscriber zeigen die Messages an
 * <p>
 * Eingabe:
 * Topic auf der Kommandozeile
 * Es wird auch eine "controlQueue" benutzt
 * Diese sollte existieren, bevor das Programm startet.
 */
public class TopicSelectors {
    final String         CONTROL_QUEUE = "controlQueue";
    String               topicName = null;
    static final String  MESSAGE_TYPES[] =
                             {"Segeln", "Snowboarden", "Business",
                              "Kino", "Konzerte", "Parties",
                               // Ende
                              "Ende"
                             };
    static final String  END_OF_MESSAGE_STREAM_TYPE =
                             MESSAGE_TYPES[MESSAGE_TYPES.length-1];
    int                  exitResult = 0;


    /**
     * Publisher
     * publiziert Messages
     * Der Typus wird zufllig gewhlt
     * Der Text erklrt den Typus
     * Client Property NewsType wird als Selektor benutzt
     *
     * Nach dem Senden der Message Typen folgt nach einer kurzen Wartezeit
     * die Ende Message.
     */
    public class Publisher extends Thread {
        final int  NUM_SUBSCRIBERS;
        final int  ARRSIZE = 6;

        public Publisher(int numSubscribers) {
            NUM_SUBSCRIBERS = numSubscribers;
        }

        /**
         * Auswhl eines Message Typs mit dem Zufallsgenerator
         * aus java.util.
         *
         * @return	Message Type String
         */
        public String chooseType() {
           int     whichMsg;
           Random  rgen = new Random();

           whichMsg = rgen.nextInt(ARRSIZE);
           return MESSAGE_TYPES[whichMsg];
        }

        /**
         * Thread starten
         */
        public void run() {
            TopicConnectionFactory  topicConnectionFactory = null;
            TopicConnection         topicConnection = null;
            TopicSession            topicSession = null;
            Topic                   topic = null;
            TopicPublisher          topicPublisher = null;
            TextMessage             message = null;
            int                     numMsgs = ARRSIZE * 5;
            String                  messageType = null;

            try {
                topicConnectionFactory =
                    Hilfsklassen.getTopicConnectionFactory();
                topicConnection =
                    topicConnectionFactory.createTopicConnection();
                topicSession = topicConnection.createTopicSession(false,
                    Session.AUTO_ACKNOWLEDGE);
                topic = Hilfsklassen.getTopic(topicName, topicSession);
            } catch (Exception e) {
                System.out.println("Connection Problem: " + e.toString());
                if (topicConnection != null) {
                    try {
                        topicConnection.close();
                    } catch (JMSException ee) {}
                }
    	        System.exit(1);
            }

            /*
             * Kreiere Publisher
             * kreiere und sende die Messages
             * sende Ende message
             */
            try {
                /*
                 * Synchronisiere mit Subscriber
                 * Alle Subscriber sollten bereit sein
                 */
                try {
                    Hilfsklassen.receiveSynchronizeMessages("[PUBLISHER THREAD]",
                                                               CONTROL_QUEUE,
                                                               NUM_SUBSCRIBERS);
                } catch (Exception e) {
                    System.out.println("[PUBLISHER THREAD]Queue fehlt vermutlich: " + e.toString());
                    if (topicConnection != null) {
                        try {
                            topicConnection.close();
                        } catch (JMSException ee) {}
                    }
    	            System.exit(1);
    	        }

                topicPublisher = topicSession.createPublisher(topic);
                message = topicSession.createTextMessage();
                for (int i = 0; i < numMsgs; i++) {
                    messageType = chooseType();
                    message.setStringProperty("NewsTyp", messageType);
                    message.setText("Nachricht " + i + ": " + messageType);
                    System.out.println("[PUBLISHER THREAD]Message Text: "
                        + message.getText());
                    topicPublisher.publish(message);
                }

                message.setStringProperty("NewsTyp", END_OF_MESSAGE_STREAM_TYPE);
                message.setText("Das war's fuer heute.");
                System.out.println("[PUBLISHER THREAD]Message Text : "
                    + message.getText());
                topicPublisher.publish(message);
            } catch (JMSException e) {
                System.out.println("Eine Exception trat auf: " + e.toString());
                exitResult = 1;
            } finally {
                if (topicConnection != null) {
                    try {
                        topicConnection.close();
                    } catch (JMSException e) {
                        exitResult = 1;
                    }
                }
            }
        }
    }

    /**
     * Subscriber
     * selektiert Messages mit dem Selektor aus dem Konstruktor
     * Ein Listener wird registriert und wartet auf passende Messages
     * Der Listener setzt die Variable done auf true, wenn alle Messages
     * empfangen wurden (Ende Message)
     */
    public class Subscriber extends Thread {
        String                  whatKind;
        int                     subscriberNumber;

        /**
         * MultipleListener implementiert das MessageListener Interface
         * dh die onMessage Method fr die Subscriber Klasse
         */
        private class MultipleListener implements MessageListener {
            final Hilfsklassen.DoneLatch  monitor =
                new Hilfsklassen.DoneLatch();

            /**
             * Anzeige der Text Message
             * Falls der NewsTyp Ende ist wird alles abgeschlossen
             *
             * @param inMessage	eintreffende Message
             */
            public void onMessage(Message inMessage) {
                TextMessage  msg = (TextMessage) inMessage;
                String       newsType;

                try {
                    System.out.println("[SUBSCRIBER_" + subscriberNumber
                                       + "_THREAD]empfaengt die Message: "
                                       + msg.getText());
                    newsType = msg.getStringProperty("NewsTyp");
                    if (newsType.equals(TopicSelectors.END_OF_MESSAGE_STREAM_TYPE)) {
                        System.out.println("[SUBSCRIBER_" + subscriberNumber
                             + "_THREAD]empfaengt die Ende Message");
                        monitor.allDone();
                    }
                } catch(JMSException e) {
                    System.out.println("[SUBSCRIBER]Eine Exception trat in onMessage()auf: "
                                       + e.toString());
                }
            }
        }

        /**
         * Konstruktor
         * Message Typ wird als Parameter bergeben
         * auf diesen Typus hrt der Subscriber
         *
         * @param str	String aus MESSAGE_TYPES
         * @param num	Index im Subscriber Array
         */
        public Subscriber(String str, int num) {
            whatKind = str;
            subscriberNumber = num + 1;
        }

        /**
         * Thread starten
         */
        public void run() {
            TopicConnectionFactory  topicConnectionFactory = null;
            TopicConnection         topicConnection = null;
            TopicSession            topicSession = null;
            Topic                   topic = null;
            String                  selector = null;
            TopicSubscriber         topicSubscriber = null;
            MultipleListener        multipleListener = new MultipleListener();

            try {
                topicConnectionFactory =
                    Hilfsklassen.getTopicConnectionFactory();
                topicConnection =
                    topicConnectionFactory.createTopicConnection();
                topicSession = topicConnection.createTopicSession(false,
                    Session.AUTO_ACKNOWLEDGE);
                topic = Hilfsklassen.getTopic(topicName, topicSession);
            } catch (Exception e) {
                System.out.println("Connection Problem: " + e.toString());
                if (topicConnection != null) {
                    try {
                        topicConnection.close();
                    } catch (JMSException ee) {}
                }
    	        System.exit(1);
            }

            /*
             * Subscriber zum Message Selektor kreieren
             * Starten der Message Delivery.
             * Senden der sync Message an den Publisher (der wartet)
             * Warten auf Messages
             * Anzeige der Messages durch den Listener
             */
            try {
                selector = new String("NewsTyp = '" + whatKind + "'" +
                                      " ODER NewsTyp = '" + END_OF_MESSAGE_STREAM_TYPE + "'");
                System.out.println("[SUBSCRIBER_" + subscriberNumber
                                    + "_THREAD]Selector ist \"" + selector + "\"");
                topicSubscriber =
                    topicSession.createSubscriber(topic, selector, false);
                topicSubscriber.setMessageListener(multipleListener);
                topicConnection.start();

                // Let publisher know that subscriber is ready.
                try {
                    Hilfsklassen.sendSynchronizeMessage("[SUBSCRIBER_"
                                                            + subscriberNumber
                                                            + "_THREAD]",
                                                            CONTROL_QUEUE);
                } catch (Exception e) {
                    System.out.println("Die Queue fehlt vermutlich: " + e.toString());
                    if (topicConnection != null) {
                        try {
                            topicConnection.close();
                        } catch (JMSException ee) {}
                    }
    	            System.exit(1);
    	        }

                /*
                 * asynchrones Verarbeiten der zum Selektor passenden Messages
                 * Warten auf die Ende Message
                 */
                multipleListener.monitor.waitTillDone();
            } catch (JMSException e) {
                System.out.println("Eine Exception trat auf: " + e.toString());
                exitResult = 1;
            } finally {
                if (topicConnection != null) {
                    try {
                        topicConnection.close();
                    } catch (JMSException e) {
                        exitResult = 1;
                    }
                }
            }
        }
    }

    /**
     * Kreiere ein Subscriber Objekt Array fr die Message Typen
     * plus die Ende Message
     * Kreiere Publisher
     * warte mit join() auf Thread Abschluss
     */
    public void run_threads() {
        final       int NUM_SUBSCRIBERS = 3;
        Subscriber  subscriberArray[] = new Subscriber[NUM_SUBSCRIBERS];
        Publisher   publisher = new Publisher(NUM_SUBSCRIBERS);

        subscriberArray[0] = new Subscriber(MESSAGE_TYPES[2], 0);
        subscriberArray[0].start();
        subscriberArray[1] = new Subscriber(MESSAGE_TYPES[3], 1);
        subscriberArray[1].start();
        subscriberArray[2] = new Subscriber(MESSAGE_TYPES[4], 2);
        subscriberArray[2].start();
        publisher.start();


        for (int i = 0; i < subscriberArray.length; i++) {
            try {
                subscriberArray[i].join();
            } catch (InterruptedException e) {}
        }

        try {
            publisher.join();
        } catch (InterruptedException e) {}
    }

    /**
     * Lesen des Topics aus der Kommandozeile
     *
     * @param args	Topic
     */
    public static void main(String[] args) {
        TopicSelectors  ts = new TopicSelectors();

        if (args.length != 1) {
    	    System.out.println("Usage: java TopicSelectors <topic_name>");
    	    //System.exit(1);
            ts.topicName = new String("meinTopic");
    	} else {  ts.topicName = new String(args[0]);}
        System.out.println("Topic Name ist " + ts.topicName);

    	ts.run_threads();
    	Hilfsklassen.exit(ts.exitResult);
    }
}
